"""
Functions for file resampling.

"""
from typing import Union

import numpy as np


def log_resample(
    wavelength, sampling: Union[int, float], resolution: Union[int, float]
) -> np.ndarray:
    """Resample spectrum with a given sampling per resolution element.

    Parameters
    ----------
    wavelength: np.ndarray
        Wavelength array.
    sampling: int, float
        Points to sample per resolution element
    resolution: int, float
        Instrumental resolution

    Uses faster method using log and powers of a base.
    The base is (1.0 + 1.0/(sampling*resolution).

    Almost equivalent to using
    np.logspace(np.log(wavelength)/np.log(base), np.log(wavelength)/np.log(base),
        np.log(wavelength_end / wavelength_start) / np.log(base), base).
    """
    wavelength_start = np.nanmin(wavelength)
    wavelength_end = np.nanmax(wavelength)

    # Create grid using logarithms with base of (1.0 + 1.0/(sampling*resolution))
    base = 1.0 + 1.0 / (float(sampling) * float(resolution))  # type : float
    return my_logspace(wavelength_start, wavelength_end, base, end_point=True)


def my_logspace(start, stop, base, end_point: bool = False):
    """Like np.logspace but start and stop in wavelength units.

    Parameters
    ----------
    start: float
        Starting point (in real units)
    stop: float
        End point (in real units)
    base: float
        Logarithmic base to jump between points.
    end_point: bool
        Make sure to include/go past the end point

    Returns
    -------
    logspace: ndarray
        Array of points with a spacing such that x[ii+1] = x[ii] * base
        between start and stop (or stop*base if end_point = True).

    """
    n = np.log(stop / start) / np.log(base)
    # to use the end point
    if end_point:
        n = n + 1
    powers = np.arange(np.ceil(n))
    return start * base ** powers


def log_chunks(wavelength, percent):
    """Define the bounds at which $(\Delta \lambda)/\lambda = X\%$.

    Allows spectrum to be split into chunks in which the size is X% of the given wavelength.
    This takes logarithmic steps with a base of (1+X/100).
    """
    base = 1 + percent / 100.
    wl_min = np.nanmin(wavelength)
    wl_max = np.nanmax(wavelength)

    return my_logspace(wl_min, wl_max, base, end_point=True)



from typing import List

from numpy.core.multiarray import ndarray


def read_col(filename: str) -> List[List[str]]:
    """This program reads column formatted data from a file and
    returns a list in which each sublist correspond to the line's elements.
    THE RESULT IS A LIST OF STRINGS!"""

    f = open(filename, "r")

    list_data = []

    while 1:
        line = f.readline()

        if line == "":
            break
        if line[0] == "#":
            continue

        list_data.append(line.strip().split())

    f.close()

    return list_data


def read_2col(filename: str) -> List[List[float]]:
    """The same as the previous, but returns 2 vectors, corresponding each
    one to a column.THE RESULTS ARE FLOAT PYTHON VECTORS.
    Note that in python all "float" are in fact "double-precision"."""

    list_data = read_col(filename)

    col1 = []
    col2 = []

    for i, __ in enumerate(list_data):
        # checking if the line is valid
        if list_data[i][0][0] != "#":
            col1.append(float(list_data[i][0]))
            col2.append(float(list_data[i][1]))

    return [col1, col2]


def read_3col(filename: str) -> List[List[float]]:
    """The same as the previous, but returns 3 columns."""

    list_data = read_col(filename)

    col1 = []
    col2 = []
    col3 = []

    for i, __ in enumerate(list_data):
        # checking if the line is valid
        if list_data[i][0][0] != "#":
            col1.append(float(list_data[i][0]))
            col2.append(float(list_data[i][1]))
            col3.append(float(list_data[i][2]))

    return [col1, col2, col3]


def read_4col(filename: str) -> List[List[float]]:
    """The same as the previous, but returns 4 columns."""

    list_data = read_col(filename)

    col1 = []
    col2 = []
    col3 = []
    col4 = []

    for i, __ in enumerate(list_data):
        # checking if the line is valid
        if list_data[i][0][0] != "#":
            col1.append(float(list_data[i][0]))
            col2.append(float(list_data[i][1]))
            col3.append(float(list_data[i][2]))
            col4.append(float(list_data[i][3]))

    return [col1, col2, col3, col4]


################################################################################
#
#    Functions to write files in column-separated formats
#
################################################################################


def write_2col(filename, data1, data2):
    """Writes data in 2 columns separated by tabs in a "filename" file."""

    f = open(filename, "w")

    for i, __ in enumerate(data1):
        f.write("\t" + str(data1[i]) + "\t\t" + str(data2[i]) + "\n")

    f.close()


def write_3col(filename, data1, data2, data3):
    """Writes data in 2 columns separated by tabs in a "filename" file."""

    f = open(filename, "w")

    for i, __ in enumerate(data1):
        f.write(
            "\t"
            + str(data1[i])
            + "\t\t"
            + str(data2[i])
            + "\t\t"
            + str(data3[i])
            + "\n"
        )

    f.close()


def write_e_2col(filename: str, data1: ndarray, data2: ndarray) -> None:
    """Writes data in 2 columns separated by tabs in a "filename" file."""

    f = open(filename, "w")

    for i, __ in enumerate(data1):
        # f.write("\t"+str(data1[i])+"\t\t"+str(data2[i])+"\t\t"+str(data3[i])+"\n")
        f.write("\t{0:e}\t\t{1:e}\n".format(data1[i], data2[i]))

    f.close()


def write_e_3col(filename: str, data1: ndarray, data2: ndarray, data3: ndarray) -> None:
    """Writes data in 3 columns separated by tabs in a "filename" file."""

    f = open(filename, "w")

    for i, __ in enumerate(data1):
        # f.write("\t"+str(data1[i])+"\t\t"+str(data2[i])+"\t\t"+str(data3[i])+"\n")
        f.write("\t{0:e}\t\t{1:e}\t\t{2:e}\n".format(data1[i], data2[i], data3[i]))

    f.close()



"""Broadening functions

Used to convolve the spectra for
    - Stellar rotation
    - Instrumental resolution

Uses joblib.Memory to cache convolution results to skip repeated computation.
"""
from typing import Optional, Union

import multiprocess as mprocess
import numpy as np
from astropy.constants import c
from joblib import Memory
from numpy.core.multiarray import ndarray
from tqdm import tqdm

import eniric
from eniric.utilities import band_selector, mask_between, wav_selector

# Cache convolution results.
memory = Memory(location=eniric.cache["location"], verbose=0)

c_kmps = c.value / 1000


@memory.cache(ignore=["num_procs"])
def rotational_convolution(
    wavelength: ndarray,
    extended_wav: ndarray,
    extended_flux: ndarray,
    vsini: float,
    *,
    epsilon: float = 0.6,
    normalize: bool = True,
    num_procs: Optional[int] = None,
) -> ndarray:
    """Perform Rotational convolution.

    Parameters
    ----------
    wavelength: ndarray
        Wavelength to calculate convolution at.
    extended_wav: ndarray
       Wavelength extended to avoid boundary issues.
    extended_flux: ndarray
        Photon flux, at extend wavelength
    vsini: float
        Rotational velocity in km/s.
    epsilon: float (default = 0.6)
        Limb darkening coefficient
    normalize: bool (default = True)
        Area normalize the broadening kernel (corrects for unequal sampling of position).
    num_procs: int, None
        Number of processes to use with multiprocess.
        If None it is assigned to 1 less then total number of cores.
        If num_procs = 0, then multiprocess is not used.

    Returns
    -------
    convolved_flux: ndarray
        The convolved_flux evaluated at wavelength points.

    """

    def element_rot_convolution(single_wav: float) -> float:
        """Embarrassingly parallel part of rotational convolution.

        Calculates the convolution value for a single pixel.

        The parameters extended_wav, extended_flux, vsini, epsilon and normalize
        are obtained from the outer scope.

        Parameters
        ----------
        single_wav: float
            Wavelength value to calculate convolution at.

        Returns
        -------
        sum_val: float
            Sum of flux convolved for this pixel/wavelength.

        """
        # Select all values such that they are within the fwhm limits
        delta_lambda_l = single_wav * vsini / c_kmps

        index_mask = mask_between(
            extended_wav, single_wav - delta_lambda_l, single_wav + delta_lambda_l
        )

        flux_2convolve = extended_flux[index_mask]
        rotation_profile = rotation_kernel(
            extended_wav[index_mask] - single_wav,
            delta_lambda_l,
            vsini=vsini,
            epsilon=epsilon,
        )

        sum_val = np.sum(rotation_profile * flux_2convolve)

        if normalize:
            # Correct for the effect of non-equidistant sampling
            unitary_rot_val = np.sum(rotation_profile)  # Affects precision
            return sum_val / unitary_rot_val
        else:
            return sum_val

    tqdm_wav = tqdm(wavelength)

    if num_procs != 0:
        if num_procs is None:
            num_procs = mprocess.cpu_count() - 1

        mproc_pool = mprocess.Pool(processes=num_procs)

        convolved_flux = np.array(mproc_pool.map(element_rot_convolution, tqdm_wav))

        mproc_pool.close()

    else:  # num_procs == 0
        convolved_flux = np.empty_like(wavelength)  # Memory assignment
        for ii, single_wav in enumerate(tqdm_wav):
            convolved_flux[ii] = element_rot_convolution(single_wav)
    return convolved_flux


@memory.cache(ignore=["num_procs"])
def resolution_convolution(
    wavelength: ndarray,
    extended_wav: ndarray,
    extended_flux: ndarray,
    R: float,
    *,
    fwhm_lim: float = 5.0,
    normalize: bool = True,
    num_procs: Optional[int] = 1,
) -> ndarray:
    """Perform Resolution convolution.

    Parameters
    ----------
    wavelength: ndarray
        Wavelength in microns to
    extended_wav: ndarray
        Wavelength array slightly longer to avoid boundary errors.
    extended_flux: ndarray
        Photon flux
    R: float
        Resolution of instrumental profile.
    fwhm_lim: float (default = 5.0)
        FWHM limit for instrument broadening.
    normalize: bool (default = True)
        Area normalize the broadening kernels (corrects for unequal sampling of position).
    num_procs: int, None
        Number of processes to use with multiprocess.
        If None it is assigned to 1 less then total number of cores.
        If num_procs = 0, then multiprocess is not used.

    Returns
    -------
    convolved_flux: ndarray
        The convolved_flux evaluated at wavelength points.
    """

    def element_res_convolution(single_wav: float) -> float:
        """Embarrassingly parallel component of resolution convolution.

        Calculates the convolution value for a single pixel.

        The parameters extended_wav, fwhm_lim, R and normalize
        are obtained from the outer scope.

        Parameters
        ----------
        single_wav: float
            Wavelength value to calculate convolution at.

        Returns
        -------
        sum_val: float
            Sum of flux convolved for this pixel/wavelength
        """
        fwhm = single_wav / R
        # Mask of wavelength range within fwhm_lim* fwhm of wav
        fwhm_space = fwhm_lim * fwhm
        index_mask = mask_between(
            extended_wav, single_wav - fwhm_space, single_wav + fwhm_space
        )

        flux_2convolve = extended_flux[index_mask]
        # Gaussian Instrument Profile for given resolution and wavelength
        instrument_profile = unitary_gaussian(
            extended_wav[index_mask], single_wav, fwhm=fwhm
        )

        sum_val = np.sum(instrument_profile * flux_2convolve)
        if normalize:
            # Correct for the effect of convolution with non-equidistant positions
            unitary_val = np.sum(instrument_profile)  # Affects precision
            return sum_val / unitary_val
        else:
            return sum_val

    tqdm_wav = tqdm(wavelength)

    if num_procs != 0:
        if num_procs is None:
            num_procs = mprocess.cpu_count() - 1

        mproc_pool = mprocess.Pool(processes=num_procs)

        convolved_flux = np.array(mproc_pool.map(element_res_convolution, tqdm_wav))
        mproc_pool.close()

    else:  # num_procs == 0
        convolved_flux = np.empty_like(wavelength)  # Memory assignment
        for jj, single_wav in enumerate(tqdm_wav):
            convolved_flux[jj] = element_res_convolution(single_wav)
    return convolved_flux


@memory.cache(ignore=["num_procs"])
def convolution(
    wav: ndarray,
    flux: ndarray,
    vsini: float,
    R: float,
    band: str = "All",
    *,
    epsilon: float = 0.6,
    fwhm_lim: float = 5.0,
    num_procs: Optional[int] = None,
    normalize: bool = True,
):
    """Perform convolution of spectrum.

    Rotational convolution followed by a Gaussian of a specified resolution R.

    Parameters
    ----------
    wav: ndarray
        Wavelength in microns
    flux: ndarray
        Photon flux
    vsini: float
        Rotational velocity in km/s.
    R: int
        Resolution of instrumental profile.
    band: str
        Wavelength band to choose, default="All"
    epsilon: float (default = 0.6)
        Limb darkening coefficient
    fwhm_lim: float (default = 5.0)
        FWHM limit for instrument broadening.
    normalize: bool (default = True)
        Area normalize the broadening kernels (corrects for unequal sampling of position).
    num_procs: int, None
        Number of processes to use with multiprocess.
        If None it is assigned to 1 less then total number of cores.
        If num_procs = 0, then multiprocess is not used.

    Returns
    -------
    wav_band: ndarray
        Wavelength for the selected band.
    flux_band: ndarray
        Original flux for the selected band.
    flux_conv: ndarray
        Convolved flux for the selected band.
    """

    wav_band, flux_band = band_selector(wav, flux, band)

    # We need to calculate the fwhm at this value in order to set the starting point for the convolution
    fwhm_min = wav_band[0] / R  # fwhm at the extremes of vector
    fwhm_max = wav_band[-1] / R

    # performing convolution with rotation kernel
    print("Starting the Rotation convolution for vsini={0:.2f}...".format(vsini))

    delta_lambda_min = wav_band[0] * vsini / c_kmps
    delta_lambda_max = wav_band[-1] * vsini / c_kmps

    # widest wavelength bin for the rotation convolution
    lower_lim = wav_band[0] - delta_lambda_min - fwhm_lim * fwhm_min
    upper_lim = wav_band[-1] + delta_lambda_max + fwhm_lim * fwhm_max
    wav_ext_rotation, flux_ext_rotation = wav_selector(wav, flux, lower_lim, upper_lim)

    # wide wavelength bin for the resolution_convolution
    lower_lim = wav_band[0] - fwhm_lim * fwhm_min
    upper_lim = wav_band[-1] + fwhm_lim * fwhm_max
    extended_wav, __ = wav_selector(wav, flux, lower_lim, upper_lim)

    # rotational convolution
    flux_conv_rot = rotational_convolution(
        extended_wav,
        wav_ext_rotation,
        flux_ext_rotation,
        vsini,
        epsilon=epsilon,
        num_procs=num_procs,
        normalize=normalize,
    )

    print("Starting the Resolution convolution...")

    flux_conv_res = resolution_convolution(
        wav_band,
        extended_wav,
        flux_conv_rot,
        R,
        fwhm_lim=fwhm_lim,
        num_procs=num_procs,
        normalize=normalize,
    )

    return wav_band, flux_band, flux_conv_res


def unitary_gaussian(
    x: Union[range, int, ndarray],
    center: Union[float, int, str],
    fwhm: Union[float, int, str],
) -> ndarray:
    """Gaussian function of area = 1.

    Parameters
    ----------
    x: array-like
        Position array
    center: float
        Central position of Gaussian
    fwhm: float
        Full Width at Half Maximum

    Returns
    -------
    result: array-like
        Result of gaussian function sampled at x values.
    """
    if not isinstance(fwhm, (np.float, np.int)):
        raise TypeError("The fwhm value is not a number, {0}".format(type(fwhm)))
    if not isinstance(center, (np.float, np.int)):
        raise TypeError("The center value is not a number, {0}".format(type(center)))
    if not isinstance(x, np.ndarray):
        raise TypeError

    sigma = np.abs(fwhm) / (2 * np.sqrt(2 * np.log(2)))
    amp = 1.0 / (sigma * np.sqrt(2 * np.pi))
    tau = -((x - center) ** 2) / (2 * (sigma ** 2))
    result = amp * np.exp(tau)

    return result


def rotation_kernel(
    delta_lambdas: ndarray, delta_lambda_l: float, vsini: float, epsilon: float
) -> ndarray:
    """Calculate the rotation kernel for a given wavelength

    Parameters
    ----------
    delta_lambdas: array
        Wavelength values selected within delta_lambda_l around central value. (check)
    delta_lambda_l: float
        FWHM of rotational broadening. (check)
    vsini: float
        Projected rotational velocity [km/s]
    epsilon: float
        Linear limb-darkening coefficient (0-1).

    Returns
    -------
        Rotational kernel

    Notes:
    Equations * from .... book.

    """
    denominator = np.pi * vsini * (1.0 - epsilon / 3.0)
    lambda_ratio_sqr = (delta_lambdas / delta_lambda_l) ** 2.0

    c1 = 2.0 * (1.0 - epsilon) / denominator
    c2 = 0.5 * np.pi * epsilon / denominator

    return c1 * np.sqrt(1.0 - lambda_ratio_sqr) + c2 * (1.0 - lambda_ratio_sqr)


def oned_circle_kernel(x: ndarray, center: float, fwhm: float):
    """Calculate the convolution kernel for a circular fiber.

    Parameters
    ----------
    x: array
        Value to evaluate kernel at.
    center: float
        Center of kernel.
    fwhm: float
        FWHM of desired kernel.

    Returns
    -------
        Collapsed circle kernel

    Notes:
    Tries to represent the broadening by the fiber of a fiber feed spectrograph.

    Artigau 2018 - stated mathematically equivalent to a cosine between -pi/2 and pi/2. This is what has tried to be created.
    """
    fwhm_scale = 2.0943951  # Numerically derived

    A = 1  # Amplitude
    B = fwhm_scale / fwhm  # Scale to give specific fwhm

    result = A * np.cos(B * (x - center))

    # Limit to main cos lobe only
    upper_xi = center + np.pi / 2 / B
    lower_xi = center - np.pi / 2 / B
    mask = mask_between(x, lower_xi, upper_xi)
    result[~mask] = 0

    return result



#!/usr/bin/env python
import os
import sys
from typing import Optional

import matplotlib.pyplot as plt
import numpy as np
from matplotlib import rc
from numpy import ndarray

import eniric
import eniric.io_module as io
from eniric.broaden import convolution
from eniric.obsolete.utilities import read_spectrum

# set stuff for latex usage
rc("text", usetex=True)

data_rep = eniric.paths["phoenix_dat"]
results_dir = eniric.paths["results"]
resampled_dir = eniric.paths["resampled"]

# models form PHOENIX-ACES
M0_ACES = data_rep + "lte03900-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes_wave.dat"
M3_ACES = data_rep + "lte03500-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes_wave.dat"
M6_ACES = data_rep + "lte02800-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes_wave.dat"
M9_ACES = data_rep + "lte02600-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes_wave.dat"


def run_convolutions(spectrum_string: str, band: str) -> None:
    """
    Runs the convolutions for a set of spectra in batch
    """
    vsini = [1.0, 5.0, 10.0]
    R = [60000, 80000, 100000]

    spectrum = spectrum_string  # removed exec usage
    print(spectrum)
    print(
        "Running the convolutions for spectra of {0:s} in band {1:s}\n.".format(
            spectrum, band
        )
    )
    for vel in vsini:
        for res in R:
            convolve_spectra(spectrum, band, vel, res)


def save_convolution_results(
    filename: str, wavelength: ndarray, flux: ndarray, convolved_flux: ndarray
) -> int:
    """Saves convolution results to a file.

    Parameters
    ----------
    filename: str
    wavelength: array-like
     flux, convolved_flux
    """
    print("Saving results...")

    # Note: difference in sampling at 1.0 and 1.5 microns makes jumps
    # in the beginning of Y and H bands
    eniric.obsolete.IOmodule.write_e_3col(filename, wavelength, flux, convolved_flux)
    print("Done.")
    return 0


def convolve_spectra(
    spectrum,
    band,
    vsini,
    R,
    epsilon: float = 0.6,
    fwhm_lim: float = 5.0,
    num_procs: Optional[int] = None,
    results_dir: str = results_dir,
    normalize: bool = True,
    output_name: Optional[str] = None,
) -> int:
    """Load Spectrum, apply convolution and then save results."""
    print("Reading the data...")
    wav, flux = read_spectrum(spectrum)  # In microns and  photon flux.
    print("Done.")

    wav_band, flux_band, convolved_flux = convolution(
        wav,
        flux,
        vsini,
        R,
        band,
        epsilon=epsilon,
        fwhm_lim=fwhm_lim,
        num_procs=num_procs,
        normalize=normalize,
    )
    if not normalize:
        norm_ = "_unnormalized"
    else:
        norm_ = ""

    if output_name is None:
        name_model = name_assignment(spectrum)

        filename = "{0}Spectrum_{1}_{2}band_vsini{3:3.1f}_R{4:d}k{5}.txt".format(
            results_dir, name_model, band, vsini, R / 1000, norm_
        )
    else:
        filename = os.path.join(results_dir, output_name)

    save_convolution_results(filename, wav_band, flux_band, convolved_flux)

    return 0


###############################################################################
def name_assignment(spectrum: str):
    """
    assigns a name to the filename in which the spectrum is going to be saved
    """
    # Simplified to temperature and base in spectrum name.
    m0_aces = "lte03900"
    m3_aces = "lte03500"
    m6_aces = "lte02800"
    m9_aces = "lte02600"
    base = "PHOENIX-ACES-AGSS-COND-2011-HiRes_wave.dat"
    if (m0_aces in spectrum) and (base in spectrum):
        name = "M0-PHOENIX-ACES"
    elif (m3_aces in spectrum) and (base in spectrum):
        name = "M3-PHOENIX-ACES"
    elif (m6_aces in spectrum) and (base in spectrum):
        name = "M6-PHOENIX-ACES"
    elif (m9_aces in spectrum) and (base in spectrum):
        name = "M9-PHOENIX-ACES"
    else:
        raise ValueError("Name {0} not found!".format(spectrum))
    return name


if __name__ == "__main__":
    if len(sys.argv) == 3:
        run_convolutions(sys.argv[1], sys.argv[2])
    else:
        print("Arguments not compatible with called function.")



#!/usr/bin/env python
"""Near-Infrared radial velocity precision."""
import argparse
import itertools
import os
import re
import sys

import matplotlib.pyplot as plt
import numpy as np
from matplotlib import rc

import eniric
import eniric.io_module as io
import eniric.legacy
import eniric.obsolete.plotting_functions as plt_functions
import eniric.obsolete.snr_norm
import eniric.Qcalculator as Qcalculator
import eniric.snr_normalization as snrnorm
from eniric.atmosphere import Atmosphere
from eniric.obsolete.utilities import barycenter_shift
from eniric.utilities import band_selector, moving_average

rc("text", usetex=True)  # set stuff for latex usage

ref_choices = ["SELF"]
ref_choices.extend(eniric.bands["all"])


def _parser():
    """Take care of all the argparse stuff.

    :returns: the args
    """
    parser = argparse.ArgumentParser(
        description="Calculate radial velocity precision of model spectra."
    )

    parser.add_argument(
        "-b",
        "--bands",
        type=str,
        default="J",
        choices=eniric.bands["all"],
        help="Wavelength bands to select. Default=J.",
        nargs="+",
    )
    parser.add_argument(
        "-u",
        "--use_unshifted",
        default=False,
        action="store_true",
        help="Start with the un-doppler-shifted atmmodel.",
    )
    parser.add_argument(
        "-s", "--save", default=False, action="store_true", help="Save results to file."
    )
    parser.add_argument(
        "--snr", help="Mid-band SNR scaling. (Default=100)", default=100, type=float
    )
    parser.add_argument(
        "--ref_band",
        help="SNR reference band. Default=J. (Default=100). "
        "'self' scales each band relative to the SNR itself.",
        choices=ref_choices,
        default="J",
        type=str,
    )
    return parser.parse_args()


def main(bands="J", use_unshifted=False, save=False, snr=100, ref_band="J"):
    """Main function that calls calc_precision.

    Parameters
    ----------
    bands: str or list of str or None, Default="J"
        Band letters to use. None does the bands Z through K.
    use_unshifted: bool default=False
        Flag to start with the un-Doppler shifted atmmodel.
    save: bool
        Save results to file.

    """
    os.makedirs(eniric.paths["precision"], exist_ok=True)

    spectral_types = ["M0", "M3", "M6", "M9"]
    if "ALL" in bands:
        bands = ["Z", "Y", "J", "H", "K"]
    elif isinstance(bands, str):
        bands = [bands]

    vsini = ["1.0", "5.0", "10.0"]
    resolution = ["60k", "80k", "100k"]
    sampling = ["3"]

    results = calculate_prec(
        spectral_types,
        bands,
        vsini,
        resolution,
        sampling,
        plot_atm=False,
        plot_ste=False,
        plot_flux=False,
        paper_plots=False,
        rv_offset=0.0,
        use_unshifted=use_unshifted,
        snr=snr,
        ref_band=ref_band,
    )

    print("{Combination\t\tPrec_1\t\tPrec_2\t\tPrec_3")
    print("-" * 20)
    for key in results:
        print(
            "{0:s}\t\t{1:0.4f}\t{2:0.4f}\t{3:0.4f}".format(
                key, results[key][0], results[key][1], results[key][2]
            )
        )
    # Save precision results
    if save:
        output_filename = os.path.join(
            eniric.paths["precision"],
            "precision_results_2017_ref_band-{0}_snr-{1}.txt".format(ref_band, snr),
        )
        ids = []
        prec_1s = []
        prec_2s = []
        prec_3s = []
        for star in spectral_types:
            for band in bands:
                for vel in vsini:
                    for res in resolution:
                        id_string = "{0:s}-{1:s}-{2:.1f}-{3:s}".format(
                            star, band, float(vel), res
                        )
                        ids.append(id_string)
                        prec_1s.append(results[id_string][0].value)
                        prec_2s.append(results[id_string][1].value)
                        prec_3s.append(results[id_string][2].value)

        io.pdwrite_cols(
            output_filename,
            ids,
            prec_1s,
            prec_2s,
            prec_3s,
            header=["# id", r"prec_1 [m/s]", r"prec_2 [m/s]", r"prec_3 [m/s]"],
            float_format="%5.01f",
        )
        print("saved results to {}".format(output_filename))
    # return results


def strip_result_quantities(results):
    """Remove the units from Quantity results."""
    for key in results:
        results[key] = [
            results[key][0].value,
            results[key][1].value,
            results[key][2].value,
        ]
    return results


def calculate_prec(
    spectral_types,
    bands,
    vsini,
    resolution,
    sampling,
    plot_atm=False,
    plot_ste=False,
    plot_flux=True,
    paper_plots=True,
    rv_offset=0.0,
    use_unshifted=False,
    snr=100,
    ref_band="J",
    new=True,
    grad=True,
):
    """Calculate precisions for given combinations.

    grad: bool
        Use more precise gradient function.
    """
    # TODO: iterate over band last so that the J band normalization value can be
    # obtained first and applied to each band.

    print("using new config.yaml file here!!!!!!!!!!!!!!")
    results = {}  # creating empty dictionary for the results
    wav_plot_m0 = []  # creating empty lists for the plots
    flux_plot_m0 = []
    wav_plot_m3 = []
    flux_plot_m3 = []
    wav_plot_m6 = []
    flux_plot_m6 = []
    wav_plot_m9 = []
    flux_plot_m9 = []

    for band in bands:

        if use_unshifted:
            atmmodel = os.path.join(
                eniric.paths["atmmodel"],
                "{0}_{1}.txt".format(eniric.atmmodel["base"], band),
            )
            print("Reading atmospheric model...")
            atm = Atmosphere.from_file(atmmodel)
            wav_atm, flux_atm, std_flux_atm, mask_atm = (
                atm.wl,
                atm.transmission,
                atm.std,
                atm.mask,
            )

            print(
                (
                    "There were {0:d} unmasked pixels out of {1:d}., or {2:.1%}." ""
                ).format(
                    np.sum(mask_atm), len(mask_atm), np.sum(mask_atm) / len(mask_atm)
                )
            )

            print(
                "The model ranges from {0:4.2f} to {1:4.2f} micron.".format(
                    wav_atm[0], wav_atm[-1]
                )
            )
            print("Done.")
            print("Calculating impact of Barycentric movement on mask...")
            # mask_atm = atm.old_barycenter_shift(wav_atm, mask_atm, rv_offset=rv_offset)
            mask_atm = barycenter_shift(wav_atm, mask_atm, rv_offset=rv_offset)
        else:
            shifted_atmmodel = os.path.join(
                eniric.paths["atmmodel"],
                "{0}_{1}_bary.txt".format(eniric.atmmodel["base"], band),
            )
            print("Reading pre-doppler-shifted atmospheric model...")
            atm = Atmosphere.from_file(shifted_atmmodel)
            wav_atm, flux_atm, std_flux_atm, mask_atm = (
                atm.wl,
                atm.transmission,
                atm.std,
                atm.mask,
            )

        print("Done.")

        print(
            ("There were {0:d} unmasked pixels out of {1:d}, or {2:.1%}." "").format(
                np.sum(mask_atm), len(mask_atm), np.sum(mask_atm) / len(mask_atm)
            )
        )

        if plot_atm:
            # moved plotting code to separate code, eniric.obsolete.plotting_functions.py
            plt_functions.plot_atmosphere_model(wav_atm, flux_atm, mask_atm)

        # theoretical ratios calculation
        # wav_m0, flux_m0, wav_m3, flux_m3, wav_m6, flux_m6, wav_m9, flux_m9 = read_nIRspectra()

        iterations = itertools.product(spectral_types, vsini, resolution, sampling)
        # for star in spectral_types:
        #     for vel in vsini:
        #         for res in resolution:
        #             for smpl in sampling:
        for (star, vel, res, smpl) in iterations:
            file_to_read = (
                "Spectrum_{0}-PHOENIX-ACES_{1}band_vsini{2}_R{3}" "_res{4:2.01f}.txt"
            ).format(star, band, vel, res, float(smpl))
            # print("Working on "+file_to_read+".")
            try:
                wav_stellar, flux_stellar = io.pdread_2col(
                    os.path.join(eniric.paths["resampled"], file_to_read)
                )
            except FileNotFoundError:
                # Turn list of strings into strings without symbols  ["J", "K"] -> J K
                spectral_str = re.sub(r"[\[\]\"\',]", "", str(spectral_types))
                band_str = re.sub(r"[\[\]\"\',]", "", str(bands))
                vsini_str = re.sub(r"[\[\]\"\',]", "", str(vsini))
                res_str = re.sub(r"[\[\]\"\',]", "", str(resolution))
                sampling_str = re.sub(r"[\[\]\"\',]", "", str(sampling))

                print(
                    (
                        "\nFor just this file I suggest you run\n\tpython nIR_run.py -s {0} -b {1} -v {2} -R {3} "
                        "--sample_rate {4}\nOr for all the combinations you ran here\n\tpython nIR_run.py -s {5}"
                        " -b {6} -v {7} -R {8} --sample_rate {9}"
                        ""
                    ).format(
                        star,
                        band,
                        vel,
                        res,
                        smpl,
                        spectral_str,
                        band_str,
                        vsini_str,
                        res_str,
                        sampling_str,
                    )
                )
                raise
            if len(wav_stellar) == 0 or len(flux_stellar) == 0:
                raise Exception("The file {0} is empty".format(file_to_read))
            # Removing boundary effects
            wav_stellar = wav_stellar[2:-2]
            flux_stellar = flux_stellar[2:-2]

            # sample was left aside because only one value existed
            # TODO: Add metallicity and logg into id string
            id_string = "{0:s}-{1:s}-{2:.1f}-{3:s}".format(star, band, float(vel), res)

            # Getting the wav, flux and mask values from the atm model
            # that are the closest to the stellar wav values, see
            # https://stackoverflow.com/questions/2566412/find-nearest-value-in-numpy-array
            index_atm = np.searchsorted(wav_atm, wav_stellar)
            # replace indexes outside the array, at the very end, by the value at the very end
            # index_atm = [index if(index < len(wav_atm)) else len(wav_atm)-1 for index in index_atm]
            indx_mask = index_atm >= len(wav_atm)  # find broken indexs
            index_atm[indx_mask] = len(wav_atm) - 1  # replace with index of end.

            wav_atm_selected = wav_atm[index_atm]
            flux_atm_selected = flux_atm[index_atm]
            mask_atm_selected = mask_atm[index_atm]

            # Check mask masks out deep atmosphere absorption
            if np.any(flux_atm_selected[mask_atm_selected] < 0.98):
                print(
                    "####WARNGING####\nThis absorption mask does not mask out deep atmosphere transmission!"
                )
                print(
                    "Min flux_atm_selected[mask_atm_selected] = {} < 0.98\n####".format(
                        np.min(flux_atm_selected[mask_atm_selected])
                    )
                )

            # Normalize to SNR 100 in middle of J band 1.25 micron!
            # flux_stellar = normalize_flux(flux_stellar, id_string)
            # flux_stellar = snrnorm.normalize_flux(flux_stellar, id_string, new=True)  # snr=100, ref_band="J"
            flux_stellar = eniric.obsolete.snr_norm.normalize_flux(
                flux_stellar,
                id_string,
                new=new,
                snr=snr,
                ref_band=ref_band,
                sampling=smpl,
            )

            if id_string in [
                "M0-J-1.0-100k",
                "M3-J-1.0-100k",
                "M6-J-1.0-100k",
                "M9-J-1.0-100k",
            ]:
                index_ref = np.searchsorted(
                    wav_stellar, 1.25
                )  # searching for the index closer to 1.25 micron
                snr_estimate = np.sqrt(
                    np.sum(flux_stellar[index_ref - 1 : index_ref + 2])
                )
                print(
                    "\tSanity Check: The S/N for the {0:s} reference model was of {1:4.2f}.".format(
                        id_string, snr_estimate
                    )
                )
            elif "J" in id_string:
                index_ref = np.searchsorted(
                    wav_stellar, 1.25
                )  # searching for the index closer to 1.25 micron
                snr_estimate = np.sqrt(
                    np.sum(flux_stellar[index_ref - 1 : index_ref + 2])
                )
                print(
                    "\tSanity Check: The S/N for the {0:s} non-reference model was of {1:4.2f}.".format(
                        id_string, snr_estimate
                    )
                )

            # Precision given by the first method:
            print("Performing analysis for: ", id_string)
            prec_1 = Qcalculator.rv_precision(wav_stellar, flux_stellar, grad=grad)

            # Precision as given by the second_method
            wav_stellar_chunks, flux_stellar_chunks = eniric.legacy.mask_clumping(
                wav_stellar, flux_stellar, mask_atm_selected
            )

            prec_2_old = eniric.legacy.RVprec_calc_masked(
                wav_stellar_chunks, flux_stellar_chunks, grad=grad
            )
            prec_2 = eniric.legacy.RVprec_calc_masked(
                wav_stellar, flux_stellar, mask_atm_selected, grad=grad
            )

            assert np.all(prec_2_old == prec_2)

            """
            # histogram checking
            lengths = [len(chunk) for chunk in flux_stellar_chunks_unformatted]
            n, bins, patches = plt.hist(lengths, 500, range=[0.5, 500.5], histtype='stepfilled')
            plt.title(id_string)
            plt.show()
            """

            # Precision as given by the third_method
            prec_3 = Qcalculator.rv_precision(
                wav_stellar, flux_stellar, mask=flux_atm_selected ** 2, grad=grad
            )

            # Adding Precision results to the dictionary
            results[id_string] = [prec_1, prec_2, prec_3]

            # Prepare/Do for the plotting.
            if plot_ste or plot_ste == id_string:
                plt_functions.plot_stellar_spectum(
                    wav_stellar, flux_stellar, wav_atm_selected, mask_atm_selected
                )

            plot_ids = [
                "M3-Z-1.0-100k",
                "M3-Y-1.0-100k",
                "M3-J-1.0-100k",
                "M3-H-1.0-100k",
                "M3-K-1.0-100k",
            ]

            if plot_flux and id_string in plot_ids:
                wav_plot_m0.append(wav_stellar)
                flux_plot_m0.append(flux_stellar)
            if plot_flux and id_string in plot_ids:
                wav_plot_m3.append(wav_stellar)
                flux_plot_m3.append(flux_stellar)
            if plot_flux and id_string in plot_ids:
                wav_plot_m6.append(wav_stellar)
                flux_plot_m6.append(flux_stellar)
            if plot_flux and id_string in plot_ids:
                wav_plot_m9.append(wav_stellar)
                flux_plot_m9.append(flux_stellar)

    if plot_flux:
        plt_functions.plot_nIR_flux()

    if paper_plots:
        plt_functions.plot_paper_plots()

    else:
        return results


###############################################################################
def compare_output():
    """Function that compares a spectrum prior to convolution, after, and after resampling."""
    pre_convolution = "PHOENIX_ACES_spectra/lte03900-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes_wave_CUT_nIR.dat"
    pre_wav, pre_flux = io.pdread_2col(pre_convolution)
    pre_wav = np.array(pre_wav, dtype=float) * 1.0e-4  # conversion to microns
    pre_flux = np.array(pre_flux, dtype=float) * pre_wav

    convolved = "results_new/Spectrum_M6-PHOENIX-ACES_Jband_vsini1.0_R100k.txt"
    sampled = "resampled_new/Spectrum_M6-PHOENIX-ACES_Jband_vsini1.0_R100k_res3.txt"

    conv_wav, theory_flux, conv_flux = io.pdread_3col(convolved)
    sampled_wav, sampled_flux = io.pdread_2col(sampled)

    theory_flux = np.array(theory_flux)
    conv_flux = np.array(conv_flux)

    ratio_flux = moving_average(conv_flux, 300) / moving_average(theory_flux, 300)
    ratio_flux = ratio_flux / ratio_flux[0]

    plt.figure(1)
    plt.xlabel(r"wavelength [$\mu$m])")
    plt.ylabel(r"Flux[ ] ")
    plt.plot(conv_wav, np.array(theory_flux) / theory_flux[0], color="k")
    plt.plot(conv_wav, np.array(conv_flux) / conv_flux[0], color="b")
    plt.plot(conv_wav, ratio_flux, color="g", linestyle="--")
    plt.show()
    plt.close()

    conv_flux_corrected = conv_flux / ratio_flux

    plt.figure(1)
    plt.xlabel(r"wavelength [$\mu$m])")
    plt.ylabel(r"Flux corrected[ ] ")
    plt.plot(conv_wav, np.array(theory_flux) / theory_flux[0], color="k")
    plt.plot(
        conv_wav, np.array(conv_flux_corrected) / conv_flux_corrected[0], color="b"
    )
    plt.show()
    plt.close()


def calculate_all_masked(wav_atm, mask_atm):
    """Auxiliary function to calculate masked pixels in banded parts.

    Needs the code to load the atmospheric data in for each band.

    Need to load a average_tapas file.
    barycenter correct.
    concatenate result.
    """
    # calculating the number of pixels inside the mask
    wav_Z, mask_Z = band_selector(wav_atm, mask_atm, "Z")
    wav_Y, mask_Y = band_selector(wav_atm, mask_atm, "Y")
    wav_J, mask_J = band_selector(wav_atm, mask_atm, "J")
    wav_H, mask_H = band_selector(wav_atm, mask_atm, "H")
    wav_K, mask_K = band_selector(wav_atm, mask_atm, "K")

    bands_masked = np.concatenate((mask_Z, mask_Y, mask_J, mask_H, mask_K))

    print(
        (
            "Inside the bands, there were {0:.0f} unmasked pixels out of {1:d}"
            ", or {2:.1%}."
        ).format(
            np.sum(bands_masked),
            len(bands_masked),
            np.sum(bands_masked) / len(bands_masked),
        )
    )


###############################################################################

if __name__ == "__main__":
    args = vars(_parser())
    opts = {k: args[k] for k in args}
    sys.exit(main(**opts))



"""Module with functions that have precision corrections."""
from typing import Dict


def correct_artigau_2018(band: str) -> float:
    """Apply Artigau 2018 nominal Barnard's Star corrections.

    Table 3: Multiplicative correction factors to be applied
    on the RV precision derived from stellar models.
    These values correspond to the square-root of
    the flux-weighted mean Q ratio between observation
    and models for each bandpass. The nominal values are
    for a comparison with the default model described here,
    but we also explore the impact of other physical parameter
    choices.

             |Nominal|  Variants
    [Fe/H]   |−0.5   |  0.0
    log g    | 5.0   |        5.5
    Teff (K) | 3200  |              3400    3000
    ----------------------------------------
    g        |  0.66 | 0.63  0.69  0.82   0.51
    r        |  0.82 | 0.76  0.84  1.08   0.60
    i        |  0.94 | 0.81  0.99  1.26   0.73
    z        |  1.27 | 1.09  1.20  1.82   0.96
    Y        |  0.29 | 0.30  0.27  0.30   0.25
    J        |  0.38 | 0.40  0.31  0.54   0.37
    H        |  1.37 | 0.95  1.82  1.42   1.23
    K        |  1.47 | 1.06  2.00  1.66   1.27

    Usage:
      Multiply by model precisions to get to real corrections.
    """

    corrections = {
        "g": 0.66,
        "r": 0.82,
        "i": 0.94,
        "Z": 1.27,
        "z": 1.27,
        "Y": 0.29,
        "J": 0.38,
        "H": 1.37,
        "K": 1.47,
    }  # type: Dict[str, float]
    try:
        return corrections[band]
    except KeyError as e:
        raise KeyError("Band '{}' does not have a valid correction key".format(band))



#!/usr/bin/env python
"""To be run on travis to generate the results data.

Compare to published results and run other tests on it.
Don't do to many.
"""
from typing import List, Tuple

import eniric
from eniric.obsolete.nIR_run import main as nir_run
from eniric.obsolete.prepare_data import main as prepare_data

if __name__ == "__main__":
    print("Eniric paths: {}".format(eniric.paths))

    # subprocess.call("python eniric_scripts/prepare_data.py -s M0 M3 M6 M9 -l 4.50 -m 0.0", shell=True)
    prepare_data(
        startype=["M0", "M3", "M6", "M9"],
        temp=[],
        logg=[4.50],
        metallicity=[0],
        alpha=[0],
    )

    parameters = [
        (["M0"], ["Z", "J"], [1], ["60k", "100K"]),
        (["M0"], ["K", "J"], [5], ["60k"]),
        (["M3"], ["Y", "J"], [5], ["80k"]),
        (["M3"], ["K", "J"], [10], ["100k"]),
        (["M6"], ["J", "H"], [10], ["100k"]),
        (["M6"], ["J", "H"], [1], ["80k"]),
        (["M9"], ["J", "K"], [1], ["80k"]),
    ]  # type: List[Tuple[List[str], List[str], List[int], List[str]]]

    for (sptype, band, vel, res) in parameters:
        # subprocess.call(["python eniric_scripts/nIR_run.py
        #       -s {0} -b {1} -R {2} -v {3}".format(sptype, band, res, vel)], shell=True)

        nir_run(startype=sptype, vsini=vel, resolution=res, band=band)



import os
import re
from genericpath import isfile
from os.path import join
from typing import Optional, Tuple, Union

import numpy as np
from astropy import constants as const
from numpy.core.multiarray import ndarray

import eniric
import eniric.obsolete.IOmodule
from eniric import io_module as io
from eniric.atmosphere import consecutive_truths
from eniric.resample import log_resample


def get_spectrum_name(
    startype: str,
    logg: Union[float, int] = 4.50,
    feh: Union[float, int] = 0.0,
    alpha: Optional[Union[int, float]] = None,
    org: bool = False,
    flux_type: str = "photon",
) -> str:
    """Return correct phoenix spectrum filename for a given spectral type.

    Based off phoenix_utils module.

    Parameters
    ----------
    flux_type: str
        Indicate which file type to try find. e.g. "photon", "wave", ("fits" Not Implemented yet)

    Returns
    -------
    spectrum_name: str
        The name of spectrum with chosen Parameters


    Ability to select logg and metallicity (feh) later on.
    org = original locations (without Z folder option)
    """
    if feh == 0:
        feh = -0.0  # make zero negative to signed integer.

    temps = {"M0": 3900, "M3": 3500, "M6": 2800, "M9": 2600}
    if (flux_type == "photon") and (not org):
        base = "PHOENIX-ACES-AGSS-COND-2011-HiRes_wave_photon.dat"
    else:
        base = "PHOENIX-ACES-AGSS-COND-2011-HiRes_wave.dat"

    # noinspection SpellCheckingInspection
    if startype in temps.keys():
        if org:
            phoenix_name = "lte{0:05d}-{1}-{2}.{3}".format(
                temps[startype], "4.50", "0.0", base
            )
        elif (alpha is not None) and (alpha != 0.0):
            if abs(alpha) > 0.2:
                raise ValueError(
                    "Warning! Alpha is outside acceptable range -0.2->0.2. (for current science case)"
                )

            phoenix_name = os.path.join(
                "Z{0:+4.1f}.Alpha={1:+5.2f}".format(feh, alpha),
                "lte{0:05d}-{1:4.2f}{2:+4.1f}.Alpha={3:+5.2f}.{4:s}".format(
                    temps[startype], logg, feh, alpha, base
                ),
            )
        else:
            phoenix_name = os.path.join(
                "Z{0:+4.1f}".format(feh),
                "lte{0:05d}-{1:4.2f}{2:+4.1f}.{3:s}".format(
                    temps[startype], logg, feh, base
                ),
            )

        spectrum_name = phoenix_name
    elif re.match(r"^[OBAFGKML][0-9]$", startype):  # Valid spectral types
        raise NotImplementedError(
            "The spectral type '{0:s}' is not implemented yet.".format(startype)
        )
    else:
        raise ValueError("'{0:s}' is not a valid spectral type.".format(startype))

    return spectrum_name


def barycenter_shift(
    wav_atm: ndarray,
    mask_atm: ndarray,
    rv_offset: float = 0.0,
    consecutive_test: bool = True,
) -> ndarray:
    """Calculating impact of Barycentric movement on mask...

    Extends the masked region to +-30 km/s due to the barycentric motion of the earth.
    """
    # Mask values to the left and right side of mask_atm.
    # To avoid indexing errors have padded with first and last values.
    mask_iminus1 = np.concatenate(
        ([mask_atm[0], mask_atm[0]], mask_atm[:-2])
    )  # padding with first value
    mask_iplus1 = np.concatenate(
        (mask_atm[2:], [mask_atm[-1], mask_atm[-1]])
    )  # padding with last value

    pixels_total = len(mask_atm)
    masked_start = pixels_total - np.sum(mask_atm)

    barycenter_rv = 3e4  # 30 km/s in m/s
    offset_rv = rv_offset * 1.0e3  # Convert to m/s

    # Doppler shift  applied to the vectors
    delta_lambdas = wav_atm * barycenter_rv / const.c.value
    offset_lambdas = wav_atm * offset_rv / const.c.value  # offset lambda

    # Doppler shift limits of each pixel
    wav_lower_barys = wav_atm + offset_lambdas - delta_lambdas
    wav_upper_barys = wav_atm + offset_lambdas + delta_lambdas

    mask_atm_30kms = np.empty_like(mask_atm, dtype=bool)

    for i, (wav_value, mask_val) in enumerate(zip(wav_atm, mask_atm)):
        """If there are 3 consecutive zeros within +/-30km/s then make the value 0."""

        # Offset_RV is the offset applied for the star RV.
        if (
            (mask_val is False)
            and (mask_iminus1[i] is False)
            and (mask_iplus1[i] is False)
            and (rv_offset == 0)
        ):  # If the mask is false and the offset is equal to zero
            """If the value and its 2 neighbours are already zero don't do the barycenter shifts"""
            mask_atm_30kms[i] = False
        else:
            # np.searchsorted is faster then the boolean masking wavelength range
            # It returns index locations to place the min/max doppler-shifted values
            slice_limits = np.searchsorted(
                wav_atm, [wav_lower_barys[i], wav_upper_barys[i]]
            )
            slice_limits = [
                index if (index < len(wav_atm)) else len(wav_atm) - 1
                for index in slice_limits
            ]  # Fix searchsorted index

            mask_atm_slice = mask_atm[slice_limits[0] : slice_limits[1]]

            mask_atm_slice = np.asarray(
                mask_atm_slice, dtype=bool
            )  # Insure boolean dtype

            if consecutive_test:
                # Make mask value False if there are 3 or more consecutive zeros in slice.
                len_consec_zeros = consecutive_truths(~mask_atm_slice)
                if np.all(
                    ~mask_atm_slice
                ):  # All pixels of slice is zeros (shouldn't get here)
                    mask_atm_30kms[i] = False
                elif np.max(len_consec_zeros) >= 3:
                    mask_atm_30kms[i] = False
                else:
                    mask_atm_30kms[i] = True
                    if np.sum(~mask_atm_slice) > 3:
                        print(
                            "There were {0}/{1} zeros in this barycentric shift but None were 3 consecutive!".format(
                                np.sum(~mask_atm_slice), len(mask_atm_slice)
                            )
                        )
            else:
                mask_atm_30kms[i] = np.product(mask_atm_slice)
    masked_end = pixels_total - np.sum(mask_atm_30kms)
    print(
        (
            "New Barycentric impact affects the number of masked pixels by {0:04.1%} due to the atmospheric"
            " spectrum"
        ).format((masked_end - masked_start) / pixels_total)
    )
    print(
        ("Masked Pixels start = {1}, masked_pixel_end = {0}, Total = {2}").format(
            masked_end, masked_start, pixels_total
        )
    )
    return mask_atm_30kms


def read_spectrum(spec_name: str) -> Tuple[ndarray, ndarray]:
    """Function that reads a flux spectra from the database!.

    If a energy flux spectra is read then it converts it to photons.

    Parameters
    ----------
    spec_name: str
        Location and name of model spectrum file.

    Returns
    -------
    wav: array-like, float
        Wavelength in microns.
    flux_photons: array-like, float
        Photon flux.

    """
    if "_res" in spec_name or "_vsini" in spec_name:
        raise ValueError("Using wrong function to load resampled spectrum.")

    if "photon" in spec_name:
        wav_micron, flux_photons = io.pdread_2col(spec_name)
    else:
        wav, flux = io.pdread_2col(spec_name)

        wav_micron = wav * 1.0e-4  # Conversion to microns
        flux_photons = flux * wav_micron  # Convert to photons

    return wav_micron, flux_photons


results_dir = eniric.paths["results"]
resampled_dir = eniric.paths["resampled"]


def resample_allfiles(
    results_dir: Optional[str] = None, resampled_dir: Optional[str] = None
) -> int:
    """Resample all files inside results_dir folder.

    Parameters
    ----------
    results_dir: str
        Directory containing results to resample.
    resampled_dir: str
        Directory to save resampled results.
    """
    if results_dir is None:
        results_dir = eniric.paths["results"]  # type: ignore
    if resampled_dir is None:
        resampled_dir = eniric.paths["resampled"]  # type: ignore
    # Getting a list of all the files
    onlyfiles = [f for f in os.listdir(results_dir) if isfile(join(results_dir, f))]

    for spectrum_file in onlyfiles:
        if spectrum_file.endswith(".txt"):
            resampler(
                spectrum_file, results_dir=results_dir, resampled_dir=resampled_dir
            )

    return 0


def resampler(
    spectrum_name: str = "Spectrum_M0-PHOENIX-ACES_Yband_vsini1.0_R60k.txt",
    results_dir: str = results_dir,
    resampled_dir: str = resampled_dir,
    sampling: Union[int, float] = 3.0,
) -> int:
    """Resample a spectrum file by interpolation onto a grid with a
    sampling of 3 pixels per resolution element.

    Parameters
    ----------
    spectrum_name: str
        Name of spectrum.
    results_dir: str
        Directory to find the spectrum to load.
    resample_dir: str
        Directory to save the results.
    sampling: float (default=3.0)
        Sampling per pixel.
    """
    os.makedirs(resampled_dir, exist_ok=True)
    read_name = os.path.join(results_dir, spectrum_name)

    match = re.search("_R(\d{2,3})k", spectrum_name)
    if match:
        resolution = int(match.group(1)) * 1000  # type: int
    else:
        raise Exception("Did not match Resolution")

    wavelength, __, spectrum = io.pdread_3col(read_name, noheader=True)
    wav_grid = log_resample(wavelength, sampling, resolution)

    interpolated_flux = np.interp(wav_grid, wavelength, spectrum)

    output_path = [
        resampled_dir,
        "{0}_res{1:3.01f}.txt".format(spectrum_name[:-4], float(sampling)),
    ]
    filetowrite = os.path.join(*output_path)
    eniric.obsolete.IOmodule.write_e_2col(
        filetowrite, wav_grid[1:-2], interpolated_flux[1:-2]
    )  # [1:-2] for border effects

    return 0



"""This extracts the test data compressed fits.tar.gz files.

May not be needed if Starfish is able to handle .tar.gz files.
"""
import tarfile
from os.path import exists, join

import eniric


def unzip(path: str, filename: str, overwrite: bool = False):
    joint_name = join(path, filename)

    if (not exists(joint_name)) or (overwrite is True):
        with tarfile.open(join(joint_name + ".tar.gz"), "r") as tar:
            tar.extractall(path)
    else:
        print("Did not overwrite file: {0}".format(joint_name))


if __name__ == "__main__":
    path = eniric.paths["phoenix_raw"]

    unzip(path, "WAVE_PHOENIX-ACES-AGSS-COND-2011.fits")

    aces_spec = [
        "lte02600-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes.fits",
        "lte02800-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes.fits",
        "lte03500-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes.fits",
        "lte03900-4.50-0.0.PHOENIX-ACES-AGSS-COND-2011-HiRes.fits",
    ]
    for aces in aces_spec:
        unzip(join(path, "Z-0.0"), aces)

    unzip(eniric.paths["btsettl_raw"], "lte039.0-4.5-0.0a+0.0.BT-Settl.spec.fits.gz")

    # Obsolete data from make_test_data.py
    # Only include results and resampled because they take longer with convolutions.
    obsolete_data = join(eniric.paths["test_data"], "obsolete")
    unzip(obsolete_data, "results")
    unzip(obsolete_data, "resampled")



__version__ = "0.6-beta"
__all__ = [
    "atmosphere",
    "io_module",
    "Qcalculator",
    "resample",
    "snr_normalization",
    "utilities",
]

# Read the users config.yaml file.
# If it doesn't exist, print a useful help message

import os

import yaml

try:
    with open("config.yaml") as f:
        config = yaml.safe_load(f)
except FileNotFoundError as e:
    default = __file__[:-11] + "config.yaml"
    import warnings

    warnings.warn(
        "Using the default config.yaml file located at {0}. "
        "This is likely NOT what you want. Please create a similar "
        "'config.yaml' file in your current working directory.".format(default),
        UserWarning,
    )
    with open(default) as f:
        config = yaml.safe_load(f)

# Read the YAML variables into package-level dictionaries to be used by the other programs.
name = config["name"]
paths = config["paths"]
bands = config["bands"]
custom_bands = config["custom_bands"]
atmmodel = config["atmmodel"]
cache = config["cache"]

# Turn list into path
for key, value in paths.items():
    if isinstance(value, list):
        paths[key] = os.path.join(*value)

if (cache["location"] is None) or (cache["location"] == "None"):
    cache["location"] = None  # Disables caching
else:
    cache["location"] = os.path.join(*cache["location"])
